#include <unistd.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <signal.h>

#include <SDL/SDL.h>
#include <argp.h>

#include "cells_sh.h"

const char *argp_program_version = PACKAGE "-" VERSION;
const char *argp_program_bug_address = "mar <marbug@1010.co.uk>";
static char doc[] = "cellular VM viewer";
static char args_doc[] = "";
static struct argp_option options[] = {
  {"width", 'w', "WIDTH", 0, "width of the window"},
  {"height", 'h', "HEIGHT", 0, "height of the window"},
  {"boxside", 'b', "SIZE", 0,
   "size of the rectange representing one byte of memory"},
  {0}
};

struct Options
{
  int width;
  int height;
  int boxside;
};
struct Options opt;

static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct Options *opt = state->input;
  switch (key) {
  case 'w':
    opt->width = atoi (arg);
    break;
  case 'h':
    opt->height = atoi (arg);
    break;
  case 'b':
    opt->boxside = atoi (arg);
    break;
  default:
    return ARGP_ERR_UNKNOWN;
  }
  return 0;
}

static struct argp argp = { options, parse_opt, args_doc, doc };

SDL_Surface *screen;
SDL_Color colors[256];


void putpixel (SDL_Surface * surface, int x, int y, Uint32 pixel);

void
draw (struct Shared *sh, unsigned char *mem)
{
  SDL_Rect rect;
  int i, j;
  Uint32 pix;
  int vwidth = opt.width / opt.boxside, vheight = opt.height / opt.boxside;
  static int moffs = 0;
  static int mlen = 0;
  /* Lock the screen for direct access to the pixels */
  if (SDL_MUSTLOCK (screen)) {
    if (SDL_LockSurface (screen) < 0) {
      fprintf (stderr, "Can't lock screen: %s\n", SDL_GetError ());
      return;
    }
  }
  /* draw it */
  j = sh->pxp - 1;
  if (j < 0)
    j = XP_TAIL;
  if (!mlen)
    mlen = vwidth * vheight;
  if (sh->xp[j] < moffs || sh->xp[j] > moffs + mlen) {
    moffs = sh->xp[j];
    mlen = vwidth * vheight;
    if (mlen > sh->memsz - moffs)
      mlen = sh->memsz - moffs;
  }
  
  rect.w = rect.h = opt.boxside;
  for (i = 0; i < vwidth * vheight; i++) {
    int cur;
    rect.x = opt.boxside * (i % (vwidth));
    rect.y = opt.boxside * (i / (vwidth));
    if (i < mlen) {
      cur = (i + moffs > sh->cc_offs && i + moffs < sh->cc_offs + sh->cc_len);
      pix =
	SDL_MapRGB (screen->format, 0,
		    cur ? mem[i + moffs] : mem[i + moffs] / 2, 0);
    } else {
      pix = 0;
    }
    SDL_FillRect (screen, &rect, pix);
  }
  for (j = 0; j < XP_TAIL; j++) {
    unsigned int m = sh->xp[j] % sh->memsz;
    int n;
    int cur;
    if (m >= moffs && m < moffs + mlen) {
      n = j - sh->pxp;
      if (n < 0)
	n += XP_TAIL;
      rect.x = opt.boxside * ((m - moffs) % (vwidth));
      rect.y = opt.boxside * ((m - moffs) / (vwidth));
      cur = (m > sh->cc_offs && m < sh->cc_offs + sh->cc_len);
      pix =
	SDL_MapRGB (screen->format, 0, cur ? mem[m] : mem[m] / 2,
		    (int) (255.0 * (double) n / (double) XP_TAIL));
      SDL_FillRect (screen, &rect, pix);
    }
  }
  
  /* own tests */
/*  memcpy(screen->pixels,mem,sh->memsz*screen->format->BytesPerPixel);*/




  if (SDL_MUSTLOCK (screen)) {
    SDL_UnlockSurface (screen);
  }

  /* Update just the part of the display that we've changed */
   SDL_UpdateRect (screen, 0, 0,0,0);
}

void
initsdl ()
{
  int bpp,i;
  /* Initialize the SDL library */
  if (SDL_Init (SDL_INIT_VIDEO) < 0) {
    fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
    exit (1);
  }
  SDL_ShowCursor (SDL_DISABLE);

  atexit (SDL_Quit);

  bpp =
    SDL_VideoModeOK (opt.width, opt.height, 8, SDL_SWSURFACE | SDL_FULLSCREEN | SDL_HWPALETTE);
  screen =
    SDL_SetVideoMode (opt.width, opt.height, bpp,
		      SDL_SWSURFACE | SDL_FULLSCREEN | SDL_HWPALETTE);

/* deal with colours */
for (i=0;i<256;i++){
colors[i].r=0;
colors[i].g=i;
colors[i].b=0;
}

SDL_SetColors(screen, colors, 0, 256);





}

int
main (int argc, char *argv[])
{
  struct Shared *shmem;
  unsigned char *mem;
  int shmid, shmid2, semid;
  int key, x;
  int exitf = 0;
  SDL_Event event;

/*  struct sembuf semtakeover[2] = {
    {0, 0, SEM_UNDO},
    {0, 1, SEM_UNDO | IPC_NOWAIT}
  }, semgiveup[1] =
  {
    {
    0, -1, SEM_UNDO | IPC_NOWAIT}
  }; */

  if ((shmid = shmget (SHMKEY, sizeof (struct Shared), 0666)) < 0) {
    perror ("shmget(vcells)");
    exit (1);
  }
  if ((shmem = shmat (shmid, NULL, 0)) == 0) {
    perror ("shmat(vcells)");
    exit (1);
  }
  if ((shmid2 = shmget (SHMKEY2, shmem->memsz, 0666)) < 0) {
    perror ("shmget(vcells)");
    exit (1);
  }
  if ((mem = shmat (shmid2, NULL, 0)) == 0) {
    perror ("shmat(vcells)");
    exit (1);
  }

/*  if ((semid = semget (SEMKEY, 1, 0666)) < 0) {
    perror ("semget(vcells)");
    exit (1);
  }*/

  opt.width = 640;
  opt.height = 480;
  opt.boxside = 5;
  argp_parse (&argp, argc, argv, 0, 0, &opt);
  initsdl ();

  x = 0;

  while (!exitf) {
/*    semop (semid, semtakeover, 2);*/
    draw (shmem, mem);

  /*  x++;
    if (x==10){
    x=0;
    SDL_UpdateRect (screen, 0, 0, opt.width, opt.height);
    }*/

    if (SDL_PollEvent (&event)) {
      switch (event.type) {
      case SDL_KEYDOWN:
	switch (event.key.keysym.sym) {
	case SDLK_F1:
	  /* reboot */
	  shmem->reboot = 1;
	  break;
	case 'q':
	  exitf = 1;
	}
	break;
      default:
	break;
      }
    }
 /*   semop (semid, semgiveup, 1);*/
  }
  shmdt (shmem);
  return 0;
}
